// listmaker 0.1.0 (c) 1999 Graham Mitchell <mitchell@cs.leander.isd.tenet.edu>
//        24 Jul 99

// main.cc
// Main program

/*
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License (COPYING) for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software Foundation,
	Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <iostream.h>
#include <fstream.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/stat.h>
#include <string.h>
#include <time.h>

const int MAXLINE = 1024;
bool lucky( int );
void halve( int& );
void handler(int);

bool done = false;

int main(int argc, char **argv)
{
	// okay, this isn't going to be particularly great
	srand(unsigned(time(NULL)));
	signal(SIGINT,handler);
	signal(SIGHUP,handler);
	signal(SIGTERM,handler);

	// what we're going to do is open the first file and randomly select
	// songs, avoiding repeats, until 2/3 of the songs have been chosen
	// Add those songs to a big array

	// Then open the second file and randomly select songs until 1/3 of the
	// songs have been chosen.  Add these also.

	// Next file, 1/6, 1/12 and so on until the percentage is smaller than
	// the number of songs available (i.e. 1/12 songs to be picked but only
	// ten tracks).  Then you have a 50% chance of picking a song from that
	// album, then a 25% chance, and so on, but never dropping below 10%

	// Once all the possible songlist items have been picked through, the
	// list is randomly shuffled and written to disk, possibly making sure
	// that the first song is from the first album.

	// On second thought, implementation is much easier if we just do a
	// percentage chance of each song making it on the final playlist, starting 
	// at %66 percent, and halving each album, but never dropping below 1%
	// (assumes ten songs per album, so 1% chance for each song means a %10
	// chance that some song from that album will make it on)

	ifstream metafile;
	ifstream current_file;
	ofstream output_file;

	struct stat statbuf;
	unsigned long checksum = 0;
	unsigned long newcheck = 0;

	char* playlist[256];	// FIXME: resultant playlist has max 256 tracks
	char current_list[MAXLINE]  = { 0 };
	char current_track[MAXLINE] = { 0 };
	char current_path[MAXLINE]  = { 0 };
	int i=0, len, pathlen;
	int percent = 66;

	if ( argc != 3 )
	{
		cerr << "Usage: " << argv[0] << " [metalist] [outputlist]\n";
		exit(1);
	}

	do
	{
		sleep(10);

		// check status of file
		stat(argv[1], &statbuf);
		newcheck = (unsigned long) statbuf.st_mtime;

		if ( newcheck != checksum )
		{
			// input file has been changed, rebuild

			metafile.open(argv[1]);

			percent = 66;
			i=0;
			// begin main loop
			while ( metafile >> current_list )
			{
#ifdef DEBUG
				cout << "****" << current_list << ':' << percent << endl;
#endif
				len = strlen(current_list) - 1;
				while ( current_list[len] != '/' )
					--len;
				memset(current_path,0,MAXLINE);
				strncpy(current_path,current_list,len+1);
				pathlen = strlen(current_path);
				current_file.open(current_list);
				memset(current_track,0,MAXLINE);
				while ( current_file >> current_track )
				{
					if ( lucky(percent) )
					{
						// add this track
#ifdef DEBUG
						cout << '+';
#endif
						len = strlen(current_track) + pathlen;
						playlist[i] = new char[len+1];
						strcpy(playlist[i],current_path);
						strcat(playlist[i],current_track);
						++i;
					}
#ifdef DEBUG
					else
						cout << '-';
					cout << current_track << endl;
#endif
				}
				current_file.close();
				halve(percent);
			}
			metafile.close();

			// at this point, playlist should have all the items and i should be
			// the number of tracks (not the index of the last item)

			// shuffle the thing (using pointers to avoid useless strcpy)

			int a, b;
			char* temp;
			for ( len=0; len<i*20; ++len )
			{
				// pick two random slots, exchange them
				a = rand()%i;
				b = rand()%i;
				temp = playlist[a];
				playlist[a] = playlist[b];
				playlist[b] = temp;
			}

			// and finally write it out to the output file

			output_file.open(argv[2]);
			for ( len=0; len<i; ++len )
				output_file << playlist[len] << '\n';
			output_file.close();

			// now clean up and go home

			for ( len=0; len<i; ++len )
				delete [] playlist[len];

			checksum = newcheck;
		}

	} while ( !done );

	return 0;
}



bool lucky ( int percent )
{
	return ( (rand()%100+1) <= percent );
}


void halve ( int& val )
{
	val /= 2;
	if ( val == 0 ) val = 1;
}


void handler(int n)
{
	done = true;
}
